﻿Shader "Unlit/GeometryShaderExample"
{
	Properties
	{

	}
	SubShader
	{
		Tags { "RenderType"="Opaque" }

		Pass
		{
			CGPROGRAM
			#pragma vertex vert
			#pragma geometry geom
			#pragma fragment frag
            #pragma enable_d3d11_debug_symbols
            #pragma target 5.0
			
			#include "UnityCG.cginc"

			struct v2g
			{
				float4 position : SV_POSITION;
                float3 color : TEXCOORD0;
                float scale : TEXCOORD1;
			};

			struct g2f
			{
				float4 position : SV_POSITION;
                float3 color : TEXCOORD0;
                float2 uv : TEXCOORD1;
			};
			
            struct ComputeBufferData
            {
                float3 centerPosition;
                float3 color;
                float scale;
            };

            // A structured buffer allows us to have a list or array
            // of custom buffer data that we can index.
            StructuredBuffer<ComputeBufferData> _ComputeBufferData;

			v2g vert (uint instanceId : SV_InstanceID)
			{
				v2g o;

                // Index our compute buffer data via the current instance id
                // since we used an instanced draw call.
                ComputeBufferData computeBufferData =_ComputeBufferData[instanceId];
				o.position = UnityObjectToClipPos(computeBufferData.centerPosition);
                o.color = computeBufferData.color;
                o.scale = computeBufferData.scale;

				return o;
			}

			[maxvertexcount(6)]
            void geom( point v2g i[1], inout TriangleStream<g2f> triangleStream)
            {
				g2f o;
				
                // Make sure we multiply our scale's x component by the aspect ratio
                // so our quad won't be squashed.
                const float aspectRatio = _ScreenParams.y / _ScreenParams.x;
                float2 scale = float2(i[0].scale * aspectRatio, i[0].scale);

                // Currently, the point's position is in clip space.  I would like to get it
                // in normalized device coordinate space which will be between -1 and 1 for the x and y components.
                // This makes the space a little easier.  So we need to divde each component by w.
                const float OneOverW = 1.0 / i[0].position.w;
                float3 ndc = float3(i[0].position.x * OneOverW, i[0].position.y * OneOverW, i[0].position.z * OneOverW);

                // We will generate 2 triangles to build up our quad.
                // We will add a scale offset and then multiply by w to put the position back into
                // clip space as that is the space that the fragment shader expects the position to be in.
                // I will also procedurally generate uvs for each vertex so that we can take advantage of
                // uvs in the fragement shader.

                // Left Triangle
                o.position = float4((ndc + float3(scale.x, -scale.y, 0)) * i[0].position.w, i[0].position.w);
                o.color = i[0].color;
                o.uv = float2(1,1);
                triangleStream.Append(o);

				o.position = float4((ndc + float3(-scale.x, scale.y, 0)) * i[0].position.w, i[0].position.w);
                o.color = i[0].color;
                o.uv = float2(0, 0);
				triangleStream.Append(o);

                o.position = float4((ndc + float3(-scale.x, -scale.y, 0)) * i[0].position.w, i[0].position.w);
                o.color = i[0].color;
                o.uv = float2(0, 1);
                triangleStream.Append(o);

                triangleStream.RestartStrip();

                // Right Triangle
                o.position = float4((ndc + float3(-scale.x, scale.y, 0)) * i[0].position.w, i[0].position.w);
                o.color = i[0].color;
                o.uv = float2(0, 0);
                triangleStream.Append(o);

                o.position = float4((ndc + float3(scale.x, -scale.y, 0)) * i[0].position.w, i[0].position.w);
                o.color = i[0].color;
                o.uv = float2(1, 1);
                triangleStream.Append(o);

                o.position = float4((ndc + float3(scale.x, scale.y, 0)) * i[0].position.w, i[0].position.w);
                o.color = i[0].color;
                o.uv = float2(1, 0);
                triangleStream.Append(o);

				triangleStream.RestartStrip();
            }
			
			fixed4 frag (g2f i) : SV_Target
			{
                // To show off the uvs we procedurally generated in the geometry shader, 
                // we will make the quad fade from top to bottom.
				fixed4 col = fixed4(i.uv.yyy,1) * float4(i.color.rgb, 1.0);
				return col;
			}
			ENDCG
		}
	}
}
